"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const request = require("request-promise-native");
const reqErrors = require("request-promise-native/errors");
const MCUtil = require("../MCUtil");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
const Endpoints_1 = require("../constants/Endpoints");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
exports.DEFAULT_CONNINFO = {
    host: "localhost",
    port: 9090
};
const STRING_NS = StringNamespaces_1.default.CMD_NEW_CONNECTION;
function newConnectionCmd() {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("New connection command invoked");
        // Only localhost is permitted. Uncomment this to (start to) support other hosts
        /*
        const inputOpts: vscode.InputBoxOptions = {
            prompt: Translator.t(STRING_NS, "enterMicroclimateHost"),
            value: DEFAULT_CONNINFO.host,
        };
        const hostname: string | undefined = await vscode.window.showInputBox(inputOpts);
    
        if (hostname == null) {
            // user cancelled
            return;
        }*/
        const hostname = exports.DEFAULT_CONNINFO.host;
        let tryAgain = true;
        let port;
        while (tryAgain) {
            const portStr = yield vscode.window.showInputBox({
                prompt: translator_1.default.t(STRING_NS, "enterMicroclimatePort"),
                value: exports.DEFAULT_CONNINFO.port.toString()
            });
            if (portStr == null) {
                // user cancelled
                return;
            }
            port = Number(portStr);
            if (!MCUtil.isGoodPort(port)) {
                const tryAgainBtn = translator_1.default.t(STRING_NS, "enterDifferentPortBtn");
                const result = yield vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "invalidPortNumber", { port: portStr }), tryAgainBtn);
                tryAgain = result === tryAgainBtn;
            }
            else {
                // they entered a good port, we can proceed.
                break;
            }
        }
        if (hostname != null && port != null) {
            const connInfo = {
                host: hostname,
                port: port
            };
            return tryAddConnection(connInfo);
        }
    });
}
exports.newConnectionCmd = newConnectionCmd;
/**
 * Test connecting to the given host:port.
 * If it fails, display a message to the user and allow them to either try to connect again with the same info,
 * or start the 'wizard' from the beginning to enter a new host/port.
 */
function tryAddConnection(connInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.i("TryAddConnection", connInfo);
        return new Promise((resolve) => {
            testConnection(connInfo)
                .then((connection) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.default.d("TestConnection success to " + connection.mcUri);
                // Connection succeeded, let the user know.
                // The ConnectionManager will signal the change and the UI will update accordingly.
                // Check if the user has this connection's workspace folder opened.
                let inMcWorkspace = false;
                const wsFolders = vscode.workspace.workspaceFolders;
                if (wsFolders != null) {
                    inMcWorkspace = wsFolders.find((folder) => folder.uri.fsPath.includes(connection.workspacePath.fsPath)) != null;
                }
                const successMsg = translator_1.default.t(STRING_NS, "connectionSucceeded", { connectionUri: connection.mcUri, workspacePath: connection.workspacePath.fsPath });
                Logger_1.default.d(successMsg);
                if (!inMcWorkspace) {
                    const openWsBtn = translator_1.default.t(STRING_NS, "openWorkspaceBtn");
                    // Provide a button to change their workspace to the microclimate-workspace if they wish
                    vscode.window.showInformationMessage(successMsg, openWsBtn) // non-nls
                        .then((response) => {
                        if (response === openWsBtn) {
                            vscode.commands.executeCommand(Commands_1.default.VSC_OPEN_FOLDER, connection.workspacePath);
                        }
                    });
                }
                else {
                    // The user already has the workspace open, we don't have to do it for them.
                    vscode.window.showInformationMessage(successMsg);
                }
                return resolve();
            }))
                .catch((s) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.default.w("Connection test failed with message " + s);
                const editBtn = translator_1.default.t(STRING_NS, "editConnectionBtn");
                const retryBtn = translator_1.default.t(STRING_NS, "retryConnectionBtn");
                const response = yield vscode.window.showErrorMessage(s, editBtn, retryBtn);
                if (response === editBtn) {
                    // start again from the beginning
                    return newConnectionCmd();
                }
                else if (response === retryBtn) {
                    // try to connect with the same host:port
                    return tryAddConnection(connInfo);
                }
            }));
        });
    });
}
exports.tryAddConnection = tryAddConnection;
// Return value resolves to a user-friendly message or error, ie "connection to $url succeeded"
function testConnection(connInfo) {
    return __awaiter(this, void 0, void 0, function* () {
        const uri = MCUtil.buildMCUrl(connInfo);
        const envUri = uri.with({ path: Endpoints_1.default.ENVIRONMENT });
        const connectTimeout = 2500;
        return new Promise((resolve, reject) => {
            request.get(envUri.toString(), { json: true, timeout: connectTimeout })
                .then((microclimateData) => {
                // Connected successfully
                return resolve(onSuccessfulConnection(uri, connInfo.host, microclimateData));
            })
                .catch((err) => {
                Logger_1.default.i(`Request fail - ${err}`);
                if (err instanceof reqErrors.RequestError) {
                    return reject(translator_1.default.t(STRING_NS, "connectFailed", { uri: uri }));
                }
                return reject(err.toString());
            });
        });
    });
}
// microclimate_version and workspace_location were both added in Microclimate 18.09
// Portal Restart API improvement was added in 18.11
const requiredVersion = 1812;
const requiredVersionStr = "18.12"; // non-nls
const internalBuildRx = /^\d{4}_M\d+_[EI]/;
/**
 * We've determined by this point that Microclimate is running at the given URI,
 * but we have to validate now that it's a new enough version.
 */
function onSuccessfulConnection(mcUri, host, mcEnvData) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            Logger_1.default.i("Microclimate ENV data:", mcEnvData);
            if (mcEnvData == null) {
                Logger_1.default.e("Null microclimateData passed to onSuccessfulConnection");
                // fail with a generic message because this should never happen
                reject(translator_1.default.t(STRING_NS, "connectFailed", { uri: mcUri }));
            }
            const rawVersion = mcEnvData.microclimate_version;
            const rawWorkspace = mcEnvData.workspace_location;
            Logger_1.default.d("rawVersion from Microclimate is", rawVersion);
            Logger_1.default.d("rawWorkspace from Microclimate is", rawWorkspace);
            if (rawVersion == null || rawWorkspace == null) {
                Logger_1.default.e("Microclimate environment did not provide either version or workspace. Data provided is:", mcEnvData);
                return reject(translator_1.default.t(STRING_NS, "versionNotProvided", { requiredVersion: requiredVersionStr }));
            }
            let versionNum;
            if (rawVersion === "latest") { // non-nls
                // This means it's being hosted by an internal MC dev.
                // There's nothing we can do here but assume they have all the features we need.
                Logger_1.default.i("Dev version of Microclimate");
                versionNum = Number.MAX_SAFE_INTEGER;
            }
            else if (rawVersion.match(internalBuildRx) != null) {
                Logger_1.default.i("Internal build of Microclimate");
                versionNum = Number.MAX_SAFE_INTEGER;
            }
            else {
                versionNum = Number(rawVersion);
                if (isNaN(versionNum)) {
                    Logger_1.default.e("Couldn't convert provided version to Number, version is: " + rawVersion);
                    return reject(translator_1.default.t(STRING_NS, "versionNotRecognized", { rawVersion: rawVersion, requiredVersion: requiredVersionStr }));
                }
                else if (versionNum < requiredVersion) {
                    Logger_1.default.e(`Microclimate version ${versionNum} is too old.`);
                    return reject(translator_1.default.t(STRING_NS, "versionTooOld", { rawVersion: rawVersion, requiredVersion: requiredVersionStr }));
                }
            }
            const workspaceUri = vscode.Uri.file(rawWorkspace);
            ConnectionManager_1.default.instance.addConnection(mcUri, host, versionNum, workspaceUri)
                .then((newConnection) => {
                return resolve(newConnection);
            })
                .catch((err) => {
                Logger_1.default.i("New connection rejected by ConnectionManager ", err);
                return reject(err);
            });
        });
    });
}
//# sourceMappingURL=NewConnectionCmd.js.map