"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const MCUtil = require("../../MCUtil");
const Connection_1 = require("./Connection");
const NewConnectionCmd_1 = require("../../command/NewConnectionCmd");
const Logger_1 = require("../../Logger");
const Settings_1 = require("../../constants/Settings");
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
class ConnectionManager {
    constructor() {
        this._connections = [];
        this.listeners = [];
        /**
         * Call this whenever a connection is added, removed, or changed.
         */
        this.onChange = () => {
            // Logger.log(`Connection ${connection} changed`);
            this.listeners.forEach((f) => f());
        };
        const connectionInfos = ConnectionManager.loadConnections();
        Logger_1.default.i(`Loaded ${connectionInfos.length} connections from settings`);
        connectionInfos.forEach((connInfo) => NewConnectionCmd_1.tryAddConnection(connInfo));
    }
    static get instance() {
        return ConnectionManager._instance || (ConnectionManager._instance = new this());
    }
    get connections() {
        return this._connections;
    }
    addConnection(uri, host, mcVersion, workspace) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                if (this.connectionExists(uri)) {
                    return reject(translator_1.default.t(StringNamespaces_1.default.DEFAULT, "connectionAlreadyExists", { uri }));
                }
                // all validation that this connection is good must be done by this point
                const newConnection = new Connection_1.default(uri, host, mcVersion, workspace);
                Logger_1.default.i("New Connection @ " + uri);
                this._connections.push(newConnection);
                ConnectionManager.saveConnections();
                this.onChange();
                return resolve(newConnection);
            });
        });
    }
    removeConnection(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            const indexToRemove = this.connections.indexOf(connection);
            if (indexToRemove === -1) {
                Logger_1.default.e(`Request to remove connection ${connection} but it doesn't exist!`);
                return false;
            }
            connection.destroy();
            this.connections.splice(indexToRemove, 1);
            Logger_1.default.i("Removed connection", connection);
            ConnectionManager.saveConnections();
            this.onChange();
            return true;
        });
    }
    connectionExists(uri) {
        return this._connections.some((conn) => {
            return conn.mcUri.toString() === uri.toString();
        });
    }
    static loadConnections() {
        const loaded = vscode.workspace.getConfiguration(Settings_1.default.CONFIG_SECTION)
            .get(Settings_1.default.CONNECTIONS_KEY, []);
        // Logger.log("LOADED CONNECTIONS", loaded);
        return loaded;
    }
    static saveConnections() {
        return __awaiter(this, void 0, void 0, function* () {
            // We save IConnectionInfo objects since they are simpler and more readable than VSCode URIs.
            // This is a bit tough to read - For each connection, convert it to a connInfo we can save nicely.
            // If the convert fails, ignore it and log an error.
            const connectionInfos = ConnectionManager.instance.connections.reduce((result, conn) => {
                const connInfo = MCUtil.getConnInfoFrom(conn.mcUri);
                if (connInfo != null) {
                    result.push(connInfo);
                }
                else {
                    // shouldn't happen
                    Logger_1.default.e("Couldn't convert mcURI to connInfo!", conn.mcUri);
                }
                return result;
            }, []);
            Logger_1.default.i("Saving connections", connectionInfos);
            try {
                return vscode.workspace.getConfiguration(Settings_1.default.CONFIG_SECTION)
                    .update(Settings_1.default.CONNECTIONS_KEY, connectionInfos, vscode.ConfigurationTarget.Global);
            }
            catch (err) {
                const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "errorSavingConnections", { err: err.toString() });
                Logger_1.default.e(msg, err);
                vscode.window.showErrorMessage(msg);
            }
        });
    }
    /*
    public async getProjectByID(projectID: string): Promise<Project | undefined> {
        for (const conn of this.connections) {
            const proj: Project | undefined = await conn.getProjectByID(projectID);
            if (proj != null) {
                return proj;
            }
        }
        return undefined;
    }*/
    /**
     * Pass this a function to call whenever a connection is added, removed, or changed,
     * eg to trigger a tree update in the UI.
     * Test-friendly.
     */
    addOnChangeListener(callback) {
        Logger_1.default.i("Adding onChangeListener " + callback.name);
        this.listeners.push(callback);
    }
}
exports.default = ConnectionManager;
//# sourceMappingURL=ConnectionManager.js.map