"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const io = require("socket.io-client");
const vscode = require("vscode");
const Logger_1 = require("../../Logger");
const Validator_1 = require("../project/Validator");
const EventTypes_1 = require("./EventTypes");
const StartModes = require("../../constants/StartModes");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const translator_1 = require("../../constants/strings/translator");
const ProjectState_1 = require("../project/ProjectState");
const AttachDebuggerCmd_1 = require("../../command/AttachDebuggerCmd");
/**
 * Receives and reacts to socket events from Portal
 *
 * Each Connection has exactly one socket
 */
class MCSocket {
    constructor(uri, connection) {
        this.uri = uri;
        this.connection = connection;
        this.onProjectStatusChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectStatusChanged", payload);
            // I don't see any reason why these should be handled differently
            this.onProjectChanged(payload);
        });
        this.onProjectChanged = (payload) => __awaiter(this, void 0, void 0, function* () {
            // Log.d("onProjectChanged", payload);
            // Log.d(`PROJECT CHANGED name=${payload.name} appState=${payload.appStatus} ` +
            // `buildState=${payload.buildStatus} startMode=${payload.startMode}`);
            const projectID = payload.projectID;
            if (projectID == null) {
                Logger_1.default.e("No projectID in changed socket event!", payload);
                return;
            }
            const project = yield this.getProject(payload);
            if (project == null) {
                // This probably means we've got a new project - refresh everything
                Logger_1.default.i("Received projectChanged for unknown project; refreshing project list");
                this.connection.forceUpdateProjectList();
                return;
            }
            project.update(payload);
        });
        this.onProjectClosed = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.clearValidationErrors();
            this.onProjectChanged(payload);
        });
        this.onProjectDeleted = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("PROJECT DELETED", payload);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            yield project.onDelete();
            this.connection.forceUpdateProjectList();
        });
        this.onProjectRestarted = (payload) => __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("PROJECT RESTARTED", payload);
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            const projectID = payload.projectID;
            if (MCSocket.STATUS_SUCCESS !== payload.status) {
                Logger_1.default.e(`Restart failed on project ${projectID}, response is`, payload);
                let err = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { projectName: project.name });
                if (payload.error != null && payload.error.msg != null) {
                    err = payload.error.msg;
                }
                vscode.window.showErrorMessage(err);
                return;
            }
            else if (payload.ports == null) {
                // Should never happen
                const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "genericErrorProjectRestart", { projectName: project.name });
                vscode.window.showErrorMessage(msg);
                Logger_1.default.e(msg + ", payload:", payload);
                return;
            }
            Logger_1.default.d("Restart event is valid");
            const startMode = payload.startMode;
            if (!StartModes.allStartModes().includes(startMode)) {
                Logger_1.default.e(`Invalid start mode "${startMode}"`);
            }
            // This updates the ports and startMode, because those are what the payload will provide.
            project.update(payload, true);
            // Now we have to attach the debugger if it's a debug mode restart
            const isDebug = StartModes.isDebugMode(startMode);
            let restartSuccess = false;
            if (isDebug) {
                Logger_1.default.d("Attaching debugger after restart");
                try {
                    // will wait for Starting - Debug state - but this is usually not necessary,
                    // since that state will be reached before Microclimate emits the restart event
                    const debuggerAttached = yield AttachDebuggerCmd_1.default(project, true);
                    if (!debuggerAttached) {
                        vscode.window.showWarningMessage(translator_1.default.t(StringNamespaces_1.default.DEFAULT, "restartDebugAttachFailure", { startMode: StartModes.getUserFriendlyStartMode(startMode) }));
                        // restart is "done", and failed.
                        return;
                    }
                }
                catch (err) {
                    // attachDebuggerCmd shouldn't throw/reject, but just in case:
                    Logger_1.default.w("Debugger attach failed or was cancelled by user", err);
                    vscode.window.showErrorMessage(err);
                }
            }
            // Run mode, wait for project to be Started
            const terminalState = isDebug ? ProjectState_1.default.AppStates.DEBUGGING : ProjectState_1.default.AppStates.STARTED;
            Logger_1.default.d(`Waiting for terminal state ${terminalState} after restart`);
            let state;
            try {
                state = yield project.waitForState(120 * 1000, terminalState);
            }
            catch (err) {
                Logger_1.default.w(`Restart into ${startMode} mode did not complete in time, or was cancelled by user:`, err);
                vscode.window.showWarningMessage(err);
                return;
            }
            restartSuccess = state === terminalState;
            if (restartSuccess) {
                const doneRestartMsg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "restartSuccess", { projectName: project.name, startMode: StartModes.getUserFriendlyStartMode(startMode) });
                Logger_1.default.i(doneRestartMsg);
                vscode.window.showInformationMessage(doneRestartMsg);
            }
            else {
                // Either the restart failed, or the user cancelled it by initiating another restart
                const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "restartFailure", { projectName: project.name, startMode: StartModes.getUserFriendlyStartMode(startMode) });
                Logger_1.default.w(msg);
                // vscode.window.showWarningMessage(msg);
            }
        });
        this.onContainerLogs = (payload) => __awaiter(this, void 0, void 0, function* () {
            const projectID = payload.projectID;
            // const projectName = payload.projectName;
            const logContents = payload.logs;
            const log = this.connection.logManager.getAppLog(projectID);
            if (log != null) {
                log.update(logContents);
            }
        });
        this.onProjectValidated = (payload) => __awaiter(this, void 0, void 0, function* () {
            const project = yield this.getProject(payload);
            if (project == null) {
                return;
            }
            Validator_1.default.validate(project, payload);
        });
        this.getProject = (payload) => __awaiter(this, void 0, void 0, function* () {
            const projectID = payload.projectID;
            if (projectID == null) {
                // Should never happen
                Logger_1.default.e("No projectID in socket event!", payload);
                return undefined;
            }
            return this.connection.getProjectByID(projectID);
        });
        Logger_1.default.i("Creating MCSocket for URI", uri);
        this.socket = io(uri);
        this.socket.connect();
        this.socket
            .on("connect", this.connection.onConnect) // non-nls
            .on("disconnect", this.connection.onDisconnect) // non-nls
            .on(EventTypes_1.default.PROJECT_CHANGED, this.onProjectChanged)
            .on(EventTypes_1.default.PROJECT_STATUS_CHANGED, this.onProjectStatusChanged)
            .on(EventTypes_1.default.PROJECT_CLOSED, this.onProjectClosed)
            .on(EventTypes_1.default.PROJECT_DELETION, this.onProjectDeleted)
            .on(EventTypes_1.default.PROJECT_RESTART_RESULT, this.onProjectRestarted)
            .on(EventTypes_1.default.CONTAINER_LOGS, this.onContainerLogs)
            .on(EventTypes_1.default.PROJECT_VALIDATED, this.onProjectValidated);
        // We don't actually need the creation event -
        // we can create the project as needed if we get a 'changed' event for a project we don't recognize
        // .on("projectCreation",       this.onProjectCreatedOrDeleted);
    }
    /**
     * This MUST be called when the connection is removed.
     * If there are multiple sockets listening on the same connection,
     * the callbacks will be fired multiple times for the same event, which will lead to serious misbehaviour.
     */
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            this.socket.disconnect();
        });
    }
}
MCSocket.STATUS_SUCCESS = "success"; // non-nls
exports.default = MCSocket;
//# sourceMappingURL=MCSocket.js.map