"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const translator_1 = require("../../constants/strings/translator");
const StringNamespaces_1 = require("../../constants/strings/StringNamespaces");
const Logger_1 = require("../../Logger");
const STRING_NS = StringNamespaces_1.default.PROJECT;
/**
 * Wrapper for a Promise which resolves when the project reaches one of the states set in the constructor,
 * or rejects when the project does not reach one of those states in time.
 */
class ProjectPendingState {
    constructor(project, states, _resolve, _reject, timeout) {
        this.project = project;
        this.states = states;
        this._resolve = _resolve;
        this._reject = _reject;
        this.timeout = timeout;
        // if the promise this pending state is tied to has resolved or rejected
        this.fulfilled = false;
        if (states.length === 0) {
            Logger_1.default.e("Creating projectPendingState with no states!");
        }
    }
    shouldResolve(currentState) {
        return this.states.includes(currentState.appState);
    }
    resolve() {
        if (this.fulfilled) {
            // this means the project instance didn't let go of this object when it should have
            Logger_1.default.e("Pending state is being resolved or rejected for a second time!");
            return;
        }
        Logger_1.default.d("Resolving pending state(s), state is " + this.project.state.appState);
        this._resolve(this.project.state.appState);
        this.clear();
    }
    reject(timeoutMs) {
        if (this.fulfilled) {
            // this means the project instance didn't let go of this object when it should have
            Logger_1.default.e("Pending state is being rejected but has already been cleared!");
            return;
        }
        const rejectMsg = this.getRejectPendingStateMsg(timeoutMs);
        Logger_1.default.d("Rejecting pending state(s) with message:", rejectMsg);
        this._reject(rejectMsg);
        this.clear();
    }
    /**
     * Only call after resolve|reject
     */
    clear() {
        Logger_1.default.d("Clearing pending app states: " + JSON.stringify(this.states));
        clearTimeout(this.timeout);
        this.fulfilled = true;
    }
    pendingStatesAsStr() {
        if (this.states.length > 1) {
            return this.states.join(translator_1.default.t(StringNamespaces_1.default.DEFAULT, "statesSeparator"));
        }
        else {
            return this.states[0].toString();
        }
    }
    getRejectPendingStateMsg(timeoutMs) {
        let msg;
        if (timeoutMs != null) {
            msg = translator_1.default.t(STRING_NS, "didNotReachStateWithTime", { projectName: this.project.name, states: this.pendingStatesAsStr(), timeoutS: timeoutMs / 1000 });
        }
        else {
            msg = translator_1.default.t(STRING_NS, "didNotReachState", { projectName: this.project.name, states: this.pendingStatesAsStr() });
        }
        return msg;
    }
}
exports.default = ProjectPendingState;
//# sourceMappingURL=ProjectPendingState.js.map